/* Author: Damian Weber, dweber@htw-saarland.de
 * License: GPL=GNU Public License , see http://www.gnu.org/copyleft/gpl.html
 *
 **/

#include <string.h>
#include <math.h>

/* return pointer to zero-terminated static array of primes */

unsigned long *erathostenes(unsigned long upper_bound)
{
	static unsigned long *prime_table=0;

	register unsigned long i,j;

	unsigned long n_primes, np;

	char *h;     /* sieve array */


	if (upper_bound<=4) 
	{
		prime_table=new unsigned long[3];
		switch (upper_bound)
		{
		    case 4:
		    case 3: 
			prime_table[2]=0;
			prime_table[1]=3;
			prime_table[0]=2;
			break;

		    case 2: 
			prime_table[1]=0;
			prime_table[0]=2;
			break;

		    default:
			prime_table[0]=0;
		}

		return prime_table;
	}

	/* upper bound of number of primes up to upper_bound */
	n_primes=(unsigned long)((double)upper_bound/(log((double)upper_bound) - 1.5));
	// 	cout << "#malloc " << n_primes << endl;

	//	if (prime_table) delete []prime_table;

	prime_table=new unsigned long[n_primes];

	h=new char[upper_bound+1];  
	memset(h,0,upper_bound+1);        /* initialize sieve array */

	np=0;

	/* i prime => h[i]=0, 1 otherwise */
	for (i=2; i<=upper_bound/i; i++)
		if (!h[i])                            /* i prime */
		{
			prime_table[np++]=i;                /* store i in table */
			for (j=i*i; j<=upper_bound; j+=i)   /* sieve multiples of i */ 
				h[j]=1;
		}

	for (; i<=upper_bound; i++)
		if (!h[i])
			prime_table[np++]=i;

	prime_table[np]=0;

	delete []h;      
	return prime_table;

}

