/* $Id: f2npoly.h,v 1.7 2007/02/20 14:48:06 dw Exp $ */

/* Copyright (c) Damian Weber <dweber@htw-saarland.de> */
/* License: BSD-style
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
*/
#ifndef F2POLY_H
#define F2POLY_H

#include <sys/types.h>
#include "config.h"   

#include <NTL/ZZ.h>

NTL_CLIENT


const int MAX_PRIMEFACTORS=1000;

struct f2poly
{
	int n;
	unsigned long *a;
};

typedef struct f2poly f2poly_t;

#ifndef HAVE_ULONG
typedef unsigned long ulong;
#endif

void word_bit_pos(int *wp, int *posp, int i);
void set_bit(f2poly_t *fp, int i);
void clear_bit(f2poly_t *fp, int i);
int is_bit_set(const f2poly_t *fp, int i);
ulong shift_left(ulong a);
ulong shift_right(ulong a);
int find_leading_bit(const f2poly_t *fp);
void to_string(char *str,const f2poly_t *fp);
void init_poly(f2poly_t *fp);
void delete_poly(f2poly_t *fp);
int new_poly(f2poly_t *fp,int n);
int renew_poly(f2poly_t *fp,int n);
void to_f2poly(f2poly_t *fp, const char *str);
int copy_poly(f2poly_t *fp,const f2poly_t *gp);
int add(f2poly_t *hp,const f2poly_t *fp,const f2poly_t *gp);
int mul_by_X(f2poly_t *fp);
int div_by_X(f2poly_t *fp);
int mul(f2poly_t *hp,const f2poly_t *fp,const f2poly_t *gp);
int power(f2poly_t *hp,const f2poly_t *fp, int m);
int div_rem(f2poly_t *qp,f2poly_t *rp,const f2poly_t *fp,const f2poly_t *gp);
void gcd(f2poly_t *dp,const f2poly_t *fp,const f2poly_t *gp);
int irred_test(const f2poly_t *fp, f2poly_t *gp);
int square_mod(f2poly_t *hp,const f2poly_t *fp, const f2poly_t *gp);
void ul_factor(unsigned long n0, unsigned long *primes, int *expo, int *max_index);
int fast_power(f2poly_t *hp,const f2poly_t *fp, ulong m);
int generator_search(const f2poly_t *fp, f2poly_t *gp);
void irred_search(f2poly_t *fp,int deg, int all);
int fast_power_mod(f2poly_t *hp,const f2poly_t *fp, const ZZ &m,const f2poly_t *rp);
int is_generator(const f2poly_t *fp,const f2poly_t *gp,const ZZ &m,const ZZ *primes,int nf);

#endif
