// filename: plclient.c

#include <fstream>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <NTL/ZZ.h>

NTL_CLIENT

#include "f2npoly.h"

#define GET "GET\n"
#define PUT "PUT\n"
#define MAX_POLY_LEN 1024
#define MAX_EXP_LEN 1024
#define RECV_BUF_LEN 10240
#define SEND_BUF_LEN 1024
#define MAX_RECORDS 102400
#define MAX_CONFIG_LENGTH 102400

#define DEGREE 30
#define CFG_ORIG "X^30+X+1\nX^4+X+1\nX^28+X^27+X^26+X^22+X^21+X^19+X^16+X^14+X^11+X^10+X^8+X^7+X^6+1\n18\n#\n"

char poly_buf[MAX_POLY_LEN];

long rec_count = 0;
struct record
{
	f2poly_t poly;
	ZZ gamma;
	ZZ beta;
};

struct record records[MAX_RECORDS];

int read_config(char *filename, char *CFG)
{
	ifstream fin;

	fin.open(filename);
	if (!fin)
		return 0;
	fin.read(CFG,MAX_CONFIG_LENGTH);
	return fin.gcount();
}


int main(int argc, char** argv)
{
	char buf[RECV_BUF_LEN];
	int port;
	int newsock;
	int sock;
	int config_length;
	char CFG[MAX_CONFIG_LENGTH];
	ZZ gamma,beta;
	ZZ Q;
	f2poly_t field_poly;
	char *p;

	struct sockaddr_in server;

	if (argc<=1)
	{
		printf("%s: usage '%s port'\n",argv[0],argv[0]);
		return 1;
	}

	config_length=read_config("plserver.config",CFG);
	if (!config_length)
	{
		cout << "could not read configuration" << endl;
		return 1;
	}

	init_poly(&field_poly);
	p=strchr(CFG,'\n');
	if (p)
		*p=0;
	to_f2poly(&field_poly, CFG);
	if (p)
		*p='\n';

	port=atoi(argv[1]);

	server.sin_family = AF_INET;
	server.sin_port = htons(port);
	server.sin_addr.s_addr = INADDR_ANY;


	if ((sock = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
		perror("socket() failed: ");
		exit(1);        
	}

	if (bind(sock, (struct sockaddr*)(&server), sizeof(server)) == -1) {
		perror("bind() failed: ");
		exit(1);
	}

	if (listen(sock, 10) == -1) {
		perror("listen() failed: ");
		exit(1);
	}

	printf("listening on port %d\n",port);

	Q=power(to_ZZ(2),field_poly.n)-1;
	cout << "group order " << Q << endl;

	while (true)
	{
		if ((newsock=accept(sock, 0, 0)) == -1) {
			perror("accept() failed: ");
			exit(1);
		}

		/* receive */
		int recvd_bytes, pos = 0;
		while ((recvd_bytes = recv(newsock, buf + pos, RECV_BUF_LEN-pos-1, 0))>0) {
			if (recvd_bytes == -1) {
				perror("recv() failed: ");
				exit(1);
			}
			pos += recvd_bytes;     
			if (strchr(buf, '#') != 0) {
				break;
			}       
		}
		buf[pos]=0;


		char* pcmd = 0;
		if ((pcmd = strtok(buf, "\n")) == 0) {
			printf("parse error: command\n");     
			return 0;
		}

		if (strncmp(pcmd, GET, 3) == 0)
		{
			send(newsock, CFG, config_length, 0);
		} else if (strncmp(pcmd, PUT, 3) == 0) {
			char* pdp = 0;

			if ((pdp = strtok(NULL, "\n")) == 0) {
				printf("parse error: dp\n");     
				return 0;
			}
			f2poly_t dp;
			to_f2poly(&dp, pdp);
			strncpy(poly_buf, pdp,MAX_POLY_LEN);

			char* pgamma = 0;
			if ((pgamma = strtok(NULL, "\n")) == 0) {
				printf("parse error: gamma\n");     
				return 0;
			}
			//printf("pgamma=%s\n",pgamma);
			gamma = to_ZZ(pgamma);

			char* pbeta = 0;
			if ((pbeta = strtok(NULL, "\n")) == 0) {
				printf("parse error: beta\n");     
				return 0;
			}
			//printf("pbeta=%s\n",pbeta);
			beta = to_ZZ(pbeta);
			cout << "DP received: " << poly_buf << " "
				<< gamma << " " << beta << endl;

			for (int i=0; i<rec_count; i++)
			{
				to_string(poly_buf, &(records[i].poly));
				if (strcmp(pdp, poly_buf) == 0)
				{
					ZZ k1 = records[i].gamma;
					ZZ k2 = gamma;
					ZZ l1 = records[i].beta;
					ZZ l2 = beta;

					ZZ k = (k2 - k1) % Q;
					ZZ l = (l1 - l2) % Q;

					ZZ c,inv_l,t;
					XGCD(c, inv_l, t, l, Q);

					if (c==to_ZZ(1))
					{
						ZZ x = (inv_l * k) % Q;
						cout << "found x=" <<  x << endl;
						break;
					}
				}
			}
			init_poly(&(records[rec_count].poly));
			copy_poly(&(records[rec_count].poly), &dp);
			records[rec_count].beta = beta;
			records[rec_count].gamma = gamma;
			rec_count++;
		}
		shutdown(newsock,SHUT_RDWR);
		close(newsock);
	}

	return 0;
}

